import pygame
from pytmx import tmxloader
import util, scene


class Level:
    def __init__(self, filename):
        self.tiledmap = tmxloader.load_pygame(filename, pixelalpha=True)

        self.tw = self.tiledmap.tilewidth
        self.th = self.tiledmap.tileheight
        self.width = self.tiledmap.width
        self.height = self.tiledmap.height
        self.gt = self.tiledmap.getTileImage
        l = self.tiledmap.getTileLayerByName('collition')
        self.collition_layer = self.tiledmap.tilelayers.index(l)

        self.left_edge = 0
        self.top_edge = 64
        self.right_edge = self.tiledmap.width * self.tw
        self.bottom_edge = self.tiledmap.height * self.th

        self.objects =  self.tiledmap.getObjects()



class LevelSprite(pygame.sprite.Sprite):
    def __init__(self):
        self.sprite = util.cut_sprite('tiles2.png', 8, 6)


# needle
class Needle(pygame.Rect):

    def __init__(self, (x,y, w,h)):
        self.left, self.top = x, y-10
        self.w = w
        self.h = h

    def update(self, player):
        if self.colliderect(player.rect):
            player.damage(2)

class Chainsaw(LevelSprite):
    

    def __init__(self, (x,y)):
        super(Chainsaw, self).__init__()
        
        self.image = self.sprite[3][4]
        self.frames = [self.sprite[3][4], self.sprite[3][5]]
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self._r = pygame.Rect(self.rect.x, self.rect.y-10,
                             self.rect.w, self.rect.h)

        self.current_frame = 0

    def get_frame(self, frames):
        if self.current_frame  < len(frames) - 1:
            self.current_frame += 1
        else:
            self.current_frame = 0

        return self.current_frame

    def update(self, player):
        if self._r.colliderect(player.rect):
            player.damage(10)

        c_f = self.get_frame(self.frames)
        self.image = self.frames[c_f]


class MovingBlock(LevelSprite):

    def __init__(self, (x,y,w,h), (bl,br,bt,bb)):
        super(MovingBlock, self).__init__()
        
        self.image = self.get_image(w,h)
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self._r = pygame.Rect(x, y-2, w, h)

        bl,br,bt,bb = int(bl),int(br),int(bt),int(bb)  

        if bl != 0 or br != 0:
            self.boundary_l = x - (bl*16)
            self.boundary_r = x + w +(br*16)
        else:
            self.boundary_l = self.boundary_r = 0
        if bt != 0 or bb != 0:
            self.boundary_t = y - (bt*16)
            self.boundary_b = y + h +(bb*16)
        else:
            self.boundary_t = self.boundary_b = 0

    
        self.speed = 2


    def get_image(self, w, h):
        s = pygame.Surface((w, h))
        for x in range(0, w, 16):
            s.blit(self.sprite[0][0], (x, 0, 16, 16))
        return s

    def update(self, player):
        if self.boundary_r != 0 and self.boundary_l != 0:
            self.rect.x += self.speed
            self._r.x = self.rect.x

            if player.rect.colliderect(self._r):
                player.rect.left += self.speed

            if self.rect.left < self.boundary_l or self.rect.right > self.boundary_r:
                self.speed *= -1

        if self.boundary_t != 0 and self.boundary_b != 0:
            self.rect.y += self.speed
            self._r.y = self.rect.y

            if player.rect.colliderect(self.rect):
                player.rect.top += self.speed

            if self.rect.bottom > self.boundary_b or self.rect.top < self.boundary_t:
                self.speed *= -1

        
class Door(object):
    def __init__(self, x, y, w, h, l, g):
        self.g = g
        self.level = l # next level
        self.rect = pygame.Rect(x,y,w,h)
        self.lock_text = ('the door is locked, the 3 key need to open','must be somewhere around')

    def update(self, player):
        if self.rect.colliderect(player.rect):
            if player.key1 and player.key2 and player.key3:
                self.g.manager.level = self.level
                self.g.manager.go_to(scene.LevelComplete(self.level, player))
            else:
                self.g.text_to_render = self.lock_text

class Key(LevelSprite):
    def __init__(self, x, y, t, g):
        super(Key, self).__init__()
        self.g = g
        self.image = self.sprite[4][0]
        if t == '1':            
            self.image = self.sprite[4][0]
        elif t == '2':
            self.image = self.sprite[5][0]
        elif t == '3':
            self.image = self.sprite[6][0]

        self.t = t

        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y

    def update(self, player):
        if self.rect.colliderect(player.rect):
            if self.t == '1':            
                player.key1 = True
            elif self.t == '2':
                player.key2 = True
            elif self.t == '3':
                player.key3 = True
            
            self.g.fx_channel.play(self.g.sounds['pickup'])
            self.g.entities.remove(self)


