import pygame
import util, scene, enemies

class Bullet(pygame.sprite.Sprite): 
    def __init__(self, x, y, dirt, b, p, g):
        self.image = pygame.Surface((4,2))
        self.image.fill((255,0,0))
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self.boundary = self.rect.x + b
        self.speed = 8 * dirt
        self.g = g
        self.power = p

    def update(self, player):
        self.rect.left += self.speed
        if self.rect.left < self.boundary:
            for e in self.g.entities:
                if isinstance(e, enemies.Enemie):
                    if self.rect.colliderect(e.rect):
                        e.damage(self.power)
                        self.g.entities.remove(self)
        else:
            self.g.entities.remove(self)



class Player(pygame.sprite.Sprite):
    speed   = 3
    max_speed    = 8
    accel_atm    = 1

    fall_accel     = 3.75
    jump_mod       = 2.5
    jump_accel     = 25
    max_fall_speed = 30  

    def __init__(self,(x,y), g):
        super(Player, self).__init__()
        self.g = g

        self.lifes = 3
        self.hp = 100
        self.key1 = False
        self.key2 = False
        self.key3 = False
        self.vx      = x
        self.vy      = y
        self.jumping  = False
        self.falling  = False
        self.runing   = False
        self.on_block = False

        self.crouching = False
        
        self.direction = 'ltr'
        self.current_frame = 0

        self.sprite = util.cut_sprite('jabs.png', 2, 8)
        self.image  = self.sprite[0][0]
        
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self._r = pygame.Rect(x, y, self.rect.w, self.rect.h)

        self.normal_frames = [self.sprite[1][4]]
        self.run_frames = [self.sprite[0][7],
                            self.sprite[0][6],
                            self.sprite[0][5],
                            self.sprite[0][4],
                            self.sprite[0][3],
                            self.sprite[0][2],
                            self.sprite[0][1],
        ]
        self.jump_frames = [self.sprite[1][1],
                            self.sprite[1][2],
                            self.sprite[1][3],
        ]

        self.fall_frame = self.sprite[1][2]
        self.damage_frame = self.sprite[1][3]
        self.crouch_frame = self.sprite[1][1]
        self.fps = 60
        self.speed = 1

        self.bullets = []
        self.bullets_x = 24
        self.bullets_y = 26
        self.bullets_d = 200

    def get_frame(self, frames):
        if self.current_frame  < len(frames) - 1:
            self.current_frame += 1
        else:
            self.current_frame = 0

        return self.current_frame

    

    def accelerate(self, accel, speed):
        if accel != 0:
            if  ((speed < 0) and (accel > 0)) or\
                ((speed > 0 ) and (accel < 0)):
                speed = accel
            else:
                speed += accel

        if speed > self.max_speed:
            speed = self.max_speed
        if speed < -self.max_speed:
            speed = -self.max_speed

        

        return speed


        

    def collide(self, vx, vy, level, o_b):
        r = pygame.Rect(0,0, level.tw, level.th)
        for y in range(level.height):
            for x in range(level.width):
                t = level.gt(x, y, level.collition_layer)
                if t == 0:
                    continue
                else:
                    r.x = x*level.tw
                    r.y =  y*level.th
                    if self.rect.colliderect(r):
                        if vx > 0:
                            self.rect.right = r.left
                        if vx < 0:
                            self.rect.left = r.right
                        if vy > 0:
                            self.rect.bottom = r.top
                            self.on_block = True
                            self.vy = 0
                            self.jumping = False
                            self.falling = False
                        if vy < 0:
                            self.rect.top = r.bottom

        for o in o_b:
            if pygame.sprite.collide_rect(self, o):
                if vx > 0:
                    self.rect.right = o.rect.left
                if vx < 0:
                    self.rect.left = o.rect.right
                if vy > 0:
                    self.rect.bottom = o.rect.top
                    self.on_block = True
                    self.vy = 0
                    self.jumping = False
                    self.falling = False
                if vy < 0:
                    self.rect.top = o.rect.bottom       


    def damage(self, hp):
        self.hp -= hp
        self.image = self.damage_frame
        self.vy -= 6
        self.vx -= 6
        if self.hp < 0:
            self.lifes -= 1
            if self.lifes < 0:
                self.g.manager.go_to(scene.GameOver(self.g.c_level, self))
            else:
                self.g.manager.go_to(scene.LostLife(self.g.c_level, self))


    def handle_events(self, keys):
        if keys[pygame.K_LEFT]:
            self.runing = True
            self.direction = 'rtl'

        if keys[pygame.K_RIGHT]:
            self.runing = True
            self.direction = 'ltr'

        if keys[pygame.K_SPACE]:
            self.jumping = True
            self.falling = True

        if keys[pygame.K_DOWN]:
            self.crouching = True

        if keys[pygame.K_s]:
            self.bullets_y = 26
            if self.crouching:
                self.bullets_y += 24

            if self.direction == 'ltr':
                b = Bullet(self.rect.left+self.bullets_x, 
                            self.rect.top+self.bullets_y, 1, 
                            self.bullets_d , 10, self.g)
            else:
                b = Bullet(self.rect.left, 
                            self.rect.top+self.bullets_y, -1, 
                            self.bullets_d , 10, self.g)
            self.g.entities.append(b)

    def update(self,  keys, level, o_b):
        self.runing = False
        self.jumping = False
        self.falling = False
        self.crouching = False

        self.handle_events(keys)


        if self.crouching:
            
            if self.direction == 'ltr':
                self.image = self.crouch_frame
            else:
                self.image = pygame.transform.flip(self.crouch_frame,1,0)

        if not self.jumping and not self.runing and self.on_block and not self.crouching:
            c_f = self.get_frame(self.normal_frames)
            if self.direction == 'ltr':
                self.image = self.normal_frames[c_f]
            else:
                self.image = pygame.transform.flip(self.normal_frames[c_f],1,0)

        if self.runing:
            c_f = self.get_frame(self.run_frames)
            if self.direction == 'ltr':
                self.vx = self.accelerate(self.accel_atm, self.vx)
                self.image = self.run_frames[c_f]
            else:
                self.vx = self.accelerate(-self.accel_atm, self.vx)
                self.image = pygame.transform.flip(self.run_frames[c_f],1,0)
        else:
            self.vx = 0

        if self.jumping and self.on_block:
            self.vy -= self.jump_accel
            if self.direction == 'ltr':
                self.image = self.fall_frame
            else:
                self.image = pygame.transform.flip(self.fall_frame,1,0)

        if not self.on_block:
            self.vy += self.fall_accel
            
            if self.vy > self.max_fall_speed:
                self.vy = self.max_fall_speed

        if self.falling:
            if self.direction == 'ltr':
                self.image = self.fall_frame
            else:
                self.image = pygame.transform.flip(self.fall_frame,1,0)

        self.rect.left += self.vx
        self._r.left += self.vx
        self.collide(self.vx, 0, level, o_b)

        self.rect.top += self.vy
        self._r.top += self.vy
        self.on_block = False
        self.collide(0, self.vy, level, o_b)

