import pygame
import data

import player, level, enemies

SW, SH = 320, 240
TW, TH = 16, 16

CAPTION = 'pyweek-18'

SCREEN_INSTANCE = pygame.Surface((0,0))

def load_sounds(sounds):
    sounds_in = {}
    for s in sounds:
        si = pygame.mixer.Sound(data.filepath(s+'.ogg', 'sounds'))
        sounds_in[s] = si
    return sounds_in


def load_image(filename, transparent=False, alpha=False, pixel=(0,0) ):
    try: image = pygame.image.load(data.imagepath(filename))
    except pygame.error, message:
        raise SystemExit, message
    if not alpha:
        image = image.convert()
    else:
        image = image.convert_alpha()
    if transparent:
        color = image.get_at(pixel)
        image.set_colorkey(color, pygame.RLEACCEL)

    return image

def cut_sprite(filename, rows, cols):
    image = load_image(filename, True, True)
    rect = image.get_rect()
    w = rect.w / cols
    h = rect.h / rows

    sprite = range(rows)
    for i in range(rows):
        sprite[i] = range(cols)

    for r in range(rows):
        for c in range(cols):
            sprite[r][c] = image.subsurface((rect.left, rect.top, w, h))
            rect.left += w
        rect.top += h
        rect.left = 0

    return sprite

def get_tile_coords(x, y):
    px = TW * x
    py = TH * y
    return px, py

# maybe exit a better way to do this
def init_objects(objects, g):
    entities = []
    other_blocks = []
    for o in objects:
        if o.name == 'player':
            p = player.Player((o.x,o.y), g)
            entities.append(p)
        if o.name == 'needle':
            n = level.Needle((o.x,o.y, o.width, o.height))
            entities.append(n)
        if o.name == 'chainsaw':
            cs = level.Chainsaw((o.x,o.y))
            other_blocks.append(cs)
            entities.append(cs)
        if o.name == 'mblock':
            mb = level.MovingBlock((o.x,o.y, o.width, o.height),
                                    (o.bl,o.br,o.bt,o.bb))
            entities.append(mb)
            other_blocks.append(mb)

        if o.name == 'kamikaze':
            k = enemies.Kamikaze(o.x, o.y, o.bl, o.br, g)
            entities.append(k)

        if o.name == 'heli':
            h = enemies.Heli(o.x, o.y, o.bl, o.br, g)
            entities.append(h)

        if o.name == 'door':
            d = level.Door(o.x,o.y,o.width,o.height,o.level, g)
            entities.append(d)

        if o.name == 'baston':
            b = enemies.Baston(o.x,o.y, g,  o.t)
            entities.append(b)
        
        if o.name == 'key':
            k = level.Key(o.x,o.y, o.t, g)
            entities.append(k)

        if o.name == 'boss':
            B = enemies.Boss(o.x, o.y, o.bl, o.br, g)
            entities.append(B)

    return entities, p, other_blocks


