import math
import pygame
import util , player as p

class Bullet(pygame.sprite.Sprite): 
    def __init__(self, x, y, dirt, b, p, g):
        self.image = pygame.Surface((4,2))
        self.image.fill((255,120,120))
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self.boundary = self.rect.x + b
        self.speed = 8 * dirt
        self.g = g
        self.power = p

    def update(self, player):
        self.rect.left += self.speed
        if self.rect.left < self.boundary:
            for e in self.g.entities:
                if isinstance(e, p.Player):
                    if e.crouching:
                        rect = (e.rect.x, e.rect.y+32, e.rect.w, 32)
                    else:
                        rect = e.rect
                    if self.rect.colliderect(rect):
                        e.damage(self.power)
        else:
            self.g.entities.remove(self)

class Enemie(pygame.sprite.Sprite):
    def __init__(self):
        super(Enemie, self).__init__()
        self.vx = 0
        self.vy = 0
        self.speed = 0
        self.normal_frames = []
        self.run_frames = []
        self.atk_frames = []
        self.damage_frame = False
        self.current_frame = 0
        self.atking = False
        self.direction = 'ltr'
        self.hp = 0 
        self.chingar = False

    def get_frame(self, frames):
        if self.current_frame  < len(frames) - 1:
            self.current_frame += 1
        else:
            self.current_frame = 0

        return self.current_frame


    def ai_action(self):
        raise NotImplementedError

    def update(self):
        raise NotImplementedError

    def damage(self, hp):
        self.hp -= hp
        self.image = self.damage_frame
        if self.hp < 0:
            self.state = 'exploiting'
            

class Baston(Enemie):
    def __init__(self, x, y, g, t=False):
        super(Baston, self).__init__()
        self.sprite = util.cut_sprite('other.png', 2, 4)
        self.g = g
        self.normal_frames = [self.sprite[0][0],]
        self.atk_frames = [self.sprite[0][2], 
                            self.sprite[0][3], 
                            self.sprite[1][0], 
                            self.sprite[1][1]]
        self.damage_frame = self.sprite[0][1]
        self.image = self.sprite[0][0]
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        if not t:
            self.atk_rect = pygame.Rect(x-self.rect.w*10, y, self.rect.w*10, self.rect.h)
        else:
            self.atk_rect = pygame.Rect(x+self.rect.w*10, y, self.rect.w*10, self.rect.h)
        self.hp = 50
        self.speed = 3
        self.t = t
        self.state = 'normal'

    def update(self, player):
        self.image = self.sprite[0][0]
        if player.rect.colliderect(self.rect):
            self.state = 'exploiting'
            self.chingar = True
            
        if player.rect.colliderect(self.atk_rect):
            if not self.t:
                b = Bullet(self.rect.left+16,self.rect.top+5, 
                            -1, self.rect.w*2, 1,self.g)
            else:
                b = Bullet(self.rect.left+16,self.rect.top+5, 
                            1, self.rect.w*9, 1,self.g)
            self.g.entities.append(b)

        elif self.state == 'exploiting':
            c_f = self.get_frame(self.atk_frames)
            if self.direction == 'ltr':
                self.image = self.atk_frames[c_f]
            else:
                self.image = pygame.transform.flip(self.atk_frames[c_f],1,0)

            self.g.fx_channel.play(self.g.sounds['explotion5'])
            if c_f == len(self.atk_frames) -1:
                if self.chingar:
                    player.damage(40)               
                self.g.entities.remove(self)

                

        if self.t:
            self.image = pygame.transform.flip(self.image,1,0)


class Kamikaze(Enemie):
    def __init__(self, x, y, bl, br, g):
        super(Kamikaze, self).__init__()
        self.sprite = util.cut_sprite('kamikaze.png', 2, 3)
        self.normal_frames = [self.sprite[0][0],]
        self.run_frames = [self.sprite[0][1], self.sprite[0][2]]
        self.atk_frames = [self.sprite[1][0], 
                            self.sprite[1][1], 
                            self.sprite[1][2]]

        self.image = self.sprite[0][0]
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self.atk_rect = pygame.Rect(x, y, self.rect.w+(self.rect.w/2), self.rect.h)
        self.damage_frame = self.sprite[1][0]
        self.hp = 19
        self.speed = 3

        self.g = g
        self.state = 'runing'

        bl,br = int(bl),int(br)

        if bl != 0 or br != 0:
            self.boundary_l = x - (bl*16)
            self.boundary_r = x + self.rect.w +(br*16)
        else:
            self.boundary_r = 0
            self.boundary_l = 0

    def update(self, player):

        if self.state == 'runing':
            if player.rect.colliderect(self.rect):
                self.state = 'exploiting'
                self.chingar = True

            if self.boundary_r != 0 and self.boundary_l != 0:
                self.rect.x += self.speed
                self.atk_rect.x = self.rect.x
                c_f = self.get_frame(self.run_frames)
                if self.direction == 'ltr':
                    self.image = self.run_frames[c_f]
                else:
                    self.image = pygame.transform.flip(self.run_frames[c_f],1,0)

            if self.rect.left < self.boundary_l or self.rect.right > self.boundary_r:
                self.speed *= -1

            if self.speed < 0:
                self.direction == 'rtl'
            else:
                self.direction = 'ltr'


        elif self.state == 'exploiting':
            c_f = self.get_frame(self.atk_frames)
            if self.direction == 'ltr':
                self.image = self.atk_frames[c_f]
            else:
                self.image = pygame.transform.flip(self.atk_frames[c_f],1,0)

            self.g.fx_channel.play(self.g.sounds['explotion5'])
            if c_f == len(self.atk_frames) -1:                
                self.g.entities.remove(self)
                if self.chingar:
                    player.damage(10)


class Heli(Enemie):
    def __init__(self, x, y, bl, br, g):
        super(Heli, self).__init__()
        self.sprite = util.cut_sprite('heli.png', 2, 3)
        self.normal_frames = [self.sprite[0][0],]
        self.run_frames = [self.sprite[0][0], self.sprite[0][1], self.sprite[0][2]]
        self.atk_frames = [self.sprite[1][0], 
                            self.sprite[1][1], 
                            self.sprite[1][2]]

        self.image = self.sprite[0][0]
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self.atk_rect = pygame.Rect(x-self.rect.w-100, y, 200+self.rect.w, self.rect.h)
        self.damage_frame = self.sprite[1][0]
        self.hp = 30
        self.speed = 3

        self.g = g
        self.state = 'runing'

        bl,br = int(bl),int(br)

        if bl != 0 or br != 0:
            self.boundary_l = x - (bl*16)
            self.boundary_r = x + self.rect.w +(br*16)
        else:
            self.boundary_r = 0
            self.boundary_l = 0

    def update(self, player):
        if player.rect.colliderect(self.atk_rect):
            if self.speed > 0:
                b = Bullet(self.rect.left+16,self.rect.top+5, 
                            1, self.rect.w*2, 1,self.g)
            else:
                b = Bullet(self.rect.left+16,self.rect.top+5, 
                            -1, self.rect.w*9, 1,self.g)
            self.g.entities.append(b)

        if self.state == 'runing':
            if player.rect.colliderect(self.rect):
                self.state = 'exploiting'
                self.chingar = True

            if self.boundary_r != 0 and self.boundary_l != 0:
                self.rect.x += self.speed
                self.atk_rect.x = self.rect.x
                c_f = self.get_frame(self.run_frames)
                if self.direction == 'ltr':
                    self.image = self.run_frames[c_f]
                else:
                    self.image = pygame.transform.flip(self.run_frames[c_f],1,0)

            if self.rect.left < self.boundary_l or self.rect.right > self.boundary_r:
                self.speed *= -1

            if self.speed < 0:
                self.direction == 'rtl'
            else:
                self.direction = 'ltr'


        elif self.state == 'exploiting':
            c_f = self.get_frame(self.atk_frames)
            if self.direction == 'ltr':
                self.image = self.atk_frames[c_f]
            else:
                self.image = pygame.transform.flip(self.atk_frames[c_f],1,0)

            self.g.fx_channel.play(self.g.sounds['explotion5'])
            if c_f == len(self.atk_frames) -1:                
                self.g.entities.remove(self)
                if self.chingar:
                    player.damage(10)



class Boss(Enemie):
    def __init__(self, x, y, bl, br, g):
        super(Boss, self).__init__()
        self.sprite = util.cut_sprite('brain.png', 2, 8)
        self.normal_frames = [self.sprite[0][0],]
        self.run_frames = [self.sprite[0][1], 
                          self.sprite[0][2],
                          self.sprite[0][3],
                          self.sprite[0][4],
                          self.sprite[0][5],
                          self.sprite[0][6],]

        self.atk_frames = [self.sprite[1][0], 
                            self.sprite[1][1]]

        self.image = self.sprite[0][0]
        self.rect = self.image.get_rect()
        self.rect.left, self.rect.top = x, y
        self.atk_rect = pygame.Rect(x-100, y, 200+self.rect.w, self.rect.h)
        self.damage_frame = self.sprite[0][7]
        self.hp = 3000
        self.speed = 3

        self.g = g
        self.state = 'runing'

        bl,br = int(bl),int(br)

        if bl != 0 or br != 0:
            self.boundary_l = x - (bl*16)
            self.boundary_r = x + self.rect.w +(br*16)
        else:
            self.boundary_r = 0
            self.boundary_l = 0

    def update(self, player):
        if player.rect.colliderect(self.atk_rect):
            if self.speed > 0:
                b = Bullet(self.rect.left+32,self.rect.top+32, 
                            1, 500, 1,self.g)
            else:
                b = Bullet(self.rect.left+32,self.rect.top+32, 
                            -1, 500, 1,self.g)
            self.g.entities.append(b)

        if self.state == 'runing':
            if player.rect.colliderect(self.rect):
                self.state = 'exploiting'
                self.chingar = True

            if self.boundary_r != 0 and self.boundary_l != 0:
                self.rect.x += self.speed
                self.atk_rect.x = self.rect.x
                c_f = self.get_frame(self.run_frames)
                if self.direction == 'ltr':
                    self.image = self.run_frames[c_f]
                else:
                    self.image = pygame.transform.flip(self.run_frames[c_f],1,0)

            if self.rect.left < self.boundary_l or self.rect.right > self.boundary_r:
                self.speed *= -1

            if self.speed < 0:
                self.direction == 'rtl'
            else:
                self.direction = 'ltr'


        elif self.state == 'exploiting':
            c_f = self.get_frame(self.atk_frames)
            if self.direction == 'ltr':
                self.image = self.atk_frames[c_f]
            else:
                self.image = pygame.transform.flip(self.atk_frames[c_f],1,0)

            self.g.fx_channel.play(self.g.sounds['explotion5'])
            if c_f == len(self.atk_frames) -1:                
                if self.chingar:
                    player.damage(10)
                else:
                    self.g.entities.remove(self)